local PRT = LibStub("AceAddon-3.0"):NewAddon("PhenomRaidTools", "AceConsole-3.0", "AceEvent-3.0");

local AceGUI = LibStub("AceGUI-3.0")
local AceComm = LibStub("AceComm-3.0")
local AceTimer = LibStub("AceTimer-3.0")

local PhenomRaidToolsLDB = LibStub("LibDataBroker-1.1"):NewDataObject("PhenomRaidTools", {
	type = "data source",
	text = "PhenomRaidTools",
	icon = "Interface\\AddOns\\PhenomRaidTools\\Media\\Icons\\PRT.blp",
	OnClick = function() 
		PRT:Open()
	end,

	OnEnter = function()
		GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR")
		GameTooltip:AddLine("|cFF69CCF0PhenomRaidTools|r") 
		GameTooltip:AddLine("Left click to open config") 
		GameTooltip:Show() 
	end,

	OnLeave = function()
		GameTooltip:Hide()
	end})

local LibDBIcon = LibStub("LibDBIcon-1.0")


-------------------------------------------------------------------------------
-- Ace standard functions

local defaults =  {
	profile = {
		myName = UnitName("player"),
		version = "1.3.20.0", --"1.3.15.2-BETA",
		receiveMessagesFrom = "",
		enabled = true,
		testMode = false,
		testEncounterID = 9999,
		testEncounterName = "Example Encounter",
		debugMode = false,
		showOverlay = false,
		hideOverlayAfterCombat = true,
		weakAuraMode = false,

		runMode = "receiver",
		senderMode = false,
		receiverMode = true,

		overlay = {
			receiver = {
				locked = true,
				top = 450,
				left = 615,
				fontSize = 32,
				fontColor = {
					hex = "FFFFFFFF",
					r = 0,
					g = 0,
					b = 0,
					a = 1
				},
				enableSound = true,
				backdropColor = {
					r = 0,
					g = 0,
					b = 0,
					a = 0
				},
				soundFile = "Interface\\AddOns\\PhenomRaidTools\\Media\\Sounds\\ReceiveMessage.ogg",
			},

			sender = {
				top = 50,
				left = 615,
				fontSize = 14,
				backdropColor = {
					r = 0,
					g = 0,
					b = 0,
					a = 0.7
				},
				enabled = true,
				hideAfterCombat = false,
				hideDisabledTriggers = false,
			}
		},

		minimap = {
			hide = false
		},

		enabledDifficulties = {
			dungeon = {
				Normal = true,
				Heroic = true,
				Mythic = true
			},
			raid = {
				Normal = true,
				Heroic = true,
				Mythic = true
			}
		},

		triggerDefaults = {
			messageDefaults = {
				defaultWithSound = true,
			},
			rotationDefaults = {
				defaultShouldRestart = true,
				defaultIgnoreAfterActivation = false,
				defaultIgnoreDuration = 10
			},
			percentageDefaults = {
				defaultUnitID = "boss1",
				defaultCheckAgain = false,
				defaultCheckAgainAfter = 5,				
			},
			conditionDefaults = {
				additionalEvents = {},
				defaultEvent = "SPELL_CAST_START"
			}
		}, 

		encounters = {					
		},

		currentEncounter = {
			inFight = false
		},
		
		colors = {
			error = "FFff6363",
			debug = "FFdcabff",
			info = "FF6bfdff", 
			warn = "FFffc526",
			highlight = "FF69CCF0",
			disabled = "FFff1100",
			enabled = "FF00c234"
		},

		raidRoster = {
			tank1 = "Tank1",
			heal1 = "Heal1",
			dd1 = "Damager1"
		},

		addonPrefixes = { 
			weakAuraMessage = "PRT_MSG",
			addonMessage = "PRT_ADDON_MSG",
			versionRequest = "PRT_VERSION_REQ",
			versionResponse = "PRT_VERSION_RESP"
		},

		versionCheck = {

		}
	}
}

function PRT:OnInitialize()		
	self.db = LibStub("AceDB-3.0"):New("PhenomRaidToolsDB", defaults, true)
	LibDBIcon:Register("PhenomRaidTools", PhenomRaidToolsLDB, self.db.profile.minimap)

	local encounterIdx, encounter = PRT.FilterEncounterTable(self.db.profile.encounters, 9999)

	if not encounterIdx and table.empty(self.db.profile.encounters) then
		table.insert(self.db.profile.encounters, PRT.ExampleEncounter())
	end	

	-- We hold the main frame within the global addon variable 
	-- because we sometimes have to do a re-layout of the complete content
	PRT.mainWindow = nil
	PRT.mainWindowContent = nil
end

function PRT:OnEnable()
	PRT.RegisterEssentialEvents()
	
	AceComm:RegisterComm(self.db.profile.addonPrefixes.addonMessage, self.OnAddonMessage)
	AceComm:RegisterComm(self.db.profile.addonPrefixes.versionRequest, self.OnVersionRequest)
	AceComm:RegisterComm(self.db.profile.addonPrefixes.versionResponse, self.OnVersionResponse)
	C_ChatInfo.RegisterAddonMessagePrefix(PRT.db.profile.addonPrefixes.weakAuraMessage)
end

function PRT:OnDisable()
	PRT.UnregisterEssentialEvents()
end

function PRT:Open()
	if UnitAffectingCombat("player") then
		PRT.Info("Can't open during combat")
	else
		if (PRT.mainWindow and not PRT.mainWindow:IsShown()) or not PRT.mainWindow then
			PRT.SenderOverlay.Initialize(PRT.db.profile.overlay.sender)
			PRT.ReceiverOverlay.Initialize(PRT.db.profile.overlay.receiver)
			PRT.CreateMainWindow(self.db.profile)		
		end	
	end	
end

function PRT:PrintHelp()
	PRT.Info("You can use following commands:")
	PRT.Info("/prt - Will open the PRT config")
	PRT.Info("/prt versions - Will check PRT versions for each member of your group")
end

function PRT:PrintPartyOrRaidVersions()	
	local myVersion = string.gsub(PRT.db.profile.version, "[^%d]+", "")
	local myVersionN = tonumber(myVersion)

	for player, version in pairs(PRT.db.profile.versionCheck) do		
		local playerWithServer = GetUnitName(player, true)	
		local coloredName = PRT.ClassColoredName(playerWithServer)

		if version == "" or version == nil then
			PRT.Info(coloredName, ":", PRT.ColoredString("no response", PRT.db.profile.colors.disabled))
		else
			local parsedVersion = string.gsub(version, "[^%d]+", "")
			local parsedVersionN = tonumber(parsedVersion)

			if parsedVersionN and myVersionN then
				if parsedVersionN >= myVersionN then
					PRT.Info(coloredName, ":", PRT.ColoredString(version, PRT.db.profile.colors.enabled))			
				elseif parsedVersionN < myVersionN then
					PRT.Info(coloredName, ":", PRT.ColoredString(version, PRT.db.profile.colors.disabled))	
				end
			else
				PRT.Info(coloredName, ":", PRT.ColoredString(version, PRT.db.profile.colors.warn), "?")	
			end
		end
	end
end

function PRT:VersionCheck(_)
	local request = {
		type = "request",
		requestor = strjoin("-", UnitFullName("player"))
	}

	if PRT.PlayerInParty() then			
		PRT.Info("Initialize version check")
		PRT.Info("Waiting for everyone to respond...")

		self.db.profile.versionCheck = {}

		for i, name in ipairs(PRT.PartyNames()) do
			self.db.profile.versionCheck[name] = ""
		end
		
		AceComm:SendCommMessage(PRT.db.profile.addonPrefixes.versionRequest, PRT.TableToString(request), "RAID")
		AceTimer:ScheduleTimer(PRT.PrintPartyOrRaidVersions, 5)
	else
		PRT.Info("You are currently running version", PRT.ColoredString(self.db.profile.version, self.db.profile.colors.highlight))
	end
end

function PRT:ExecuteChatCommand(input)
	if input == "" or input == nil then
		PRT:Open()
	elseif input == "help" then
		PRT.PrintHelp()
	elseif input == "version" or input == "versions" then
		PRT:VersionCheck()
	else		
		PRT.PrintHelp()
	end
end


-------------------------------------------------------------------------------
-- Chat Commands

PRT:RegisterChatCommand("prt", "ExecuteChatCommand")
PRT:RegisterChatCommand("prtv", "VersionCheck")
